#include "Bullet.h"
#include "EventScore.h"
#include "EventMult.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include "Saucer.h"

Bullet::Bullet(Position hero_pos) {
  WorldManager &world_manager = WorldManager::getInstance();
  ResourceManager &rm = ResourceManager::getInstance();
  LogManager &log_manager = LogManager::getInstance();

  // register interest in STEP event, generated each frame
  world_manager.registerInterest(this, STEP_EVENT);

  // setup "saucer" sprite
  Sprite *p_temp_sprite = rm.getSprite("bullet");
  if (!p_temp_sprite) {
    log_manager.writeLog("Bullet::Bullet(): Warning! Sprite '%s' not found", "bullet");
    return;
  }

  setSprite(p_temp_sprite); // center and define bounding box
  setSpriteSlowdown(5);		

  // set other object properties
  setType("Bullet");
  move_slowdown = 1;
  move_countdown = move_slowdown;

  // set starting location, based on the hero's position passed in
  pos.setX(hero_pos.getX()+3);
  pos.setY(hero_pos.getY());
}

Bullet::~Bullet() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

// handle event
// return 0 if ignored, else 1
int Bullet::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) {
    move();
    return 1;
  }

  if (e->getType() == OUT_EVENT) {
    out();
    return 1;
  }

  if (e->getType() == COLLISION_EVENT) {
    EventCollision *p_collision_event = static_cast <EventCollision *> (e);
    hit(p_collision_event);
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

void Bullet::move() {
  move_countdown--;
  if (move_countdown > 0)
    return;
  move_countdown = move_slowdown;

  Position new_pos(pos.getX() + 1, pos.getY());
  WorldManager &game_world = WorldManager::getInstance();
  game_world.moveObj(this, new_pos);

  return;
}

// if bullet moves outside world, destroy itself, and lower the score multiplier by a bit.
void Bullet::out() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.markForDelete(this);

  EventMult *mult = new EventMult(-0.01);
  world_manager.onEvent(mult);
}

// if bullet hits saucer, mark saucer and bullet for deletion
void Bullet::hit(EventCollision *p_c) {
  WorldManager &world_manager = WorldManager::getInstance();
  if (((p_c->getObj1()->getType()) == "Saucer") ||
      ((p_c->getObj2()->getType()) == "Saucer")) {
    world_manager.markForDelete(p_c->getObj1());
    world_manager.markForDelete(p_c->getObj2());

    EventScore score;
    world_manager.onEvent(&score);
  
    EventMult *mult = new EventMult(0.1);
    world_manager.onEvent(mult);
  } else {
    // Uncomment if the player should destroy bullets it collides with
    //world_manager.markForDelete(this);
  }
}


