#include "Explosion.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"

Explosion::Explosion() {
  LogManager &log_manager = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  ResourceManager &rm = ResourceManager::getInstance();

  // register interest in STEP event, generated each frame
  world_manager.registerInterest(this, STEP_EVENT);

  // setup "explosion" sprite
  Sprite *p_temp_sprite = rm.getSprite("explosion");
  if (!p_temp_sprite) {
    log_manager.writeLog("Explosion::Explosion(): Warning! Sprite '%s' not found", "explosion");
    return;
  }
  setSprite(p_temp_sprite, true); // center and define bounding box

  // set other object properties
  setType("Explosion");
  time_to_live = p_sprite->getFrameCount();
  is_solid = false;

  altitude = 2;
}

Explosion::~Explosion() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

// handle event
// return 0 if ignored, else 1
int Explosion::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) {
    step();
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

void Explosion::step() {
  time_to_live--;
  if (time_to_live <= 0){
    WorldManager &world_manager = WorldManager::getInstance();
    world_manager.markForDelete(this);
  }
}

