#include "TextMessage.h"
#include "WorldManager.h"
#include "LogManager.h"
#include "EventStep.h"
#include "ResourceManager.h"

TextMessage::TextMessage(string message) {
  ResourceManager &resource_manager = ResourceManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();

  Sprite *p_temp_sprite = resource_manager.getSprite(message);
  if (!p_temp_sprite) {
    lm.writeLog("GameStart::GameStart(): Warning! Sprite '%s' not found", 
		message.c_str());
    return;
  }
  setSprite(p_temp_sprite);

  is_solid = false;
  setSpriteSlowdown(15);		  
  time_to_live = p_temp_sprite->getFrameCount() * 15;

  world_manager.registerInterest(this, STEP_EVENT);

  altitude = 1;

  setType("TextMessage");
  
  // put in center of screen
  pos.setX(world_manager.getBoundary().getCorner().getX() + 
	   world_manager.getBoundary().getHorizontal()/2);
  pos.setY(world_manager.getBoundary().getCorner().getY() + 
	   world_manager.getBoundary().getVertical()/2);
}

TextMessage::~TextMessage() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

// handle event
// return 0 if ignored, else 1
int TextMessage::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) {
    step();
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

// count down to end of message
void TextMessage::step() {
  time_to_live--;
  if (time_to_live <= 0) { 
    WorldManager &gw = WorldManager::getInstance();
    gw.markForDelete(this);
  }
}

